"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _default;
exports.processSync = processSync;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _pkgDir = _interopRequireDefault(require("pkg-dir"));

var _slash = _interopRequireDefault(require("slash"));

var _loaderUtils = _interopRequireDefault(require("loader-utils"));

var _magicString = _interopRequireDefault(require("magic-string"));

var _utils = require("./utils");

var VEUI_PACKAGE_NAME = 'veui';
var COMPONENTS_DIRNAME = 'components';
var EXT_TYPES = {
  SCRIPT: ['js'],
  STYLE: ['css', 'less', 'styl', 'sass', 'scss']
};
var resolveCache = {};
/**
 * webpack loader to load theme/locale modules for VEUI components
 * @param {string} content Raw file content of .vue file
 * @returns {Promise<string>} A promise that resolved with the content of patched .vue file
 */

function _default(_x) {
  return _ref.apply(this, arguments);
}
/**
 * Synchronously transform the module with a given resolveSync function.
 *
 * @param {string} content Module content
 * @param {string} file Module file path
 * @param {Object} options veui-loader options
 * @param {Function} resolveSync Resolves module path to file path
 */


function _ref() {
  _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(content) {
    var _this = this;

    var callback, loaderOptions, component, result;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            callback = this.async();
            loaderOptions = _loaderUtils["default"].getOptions(this) || {};
            component = resolveComponent(this.resourcePath, loaderOptions);

            if (component) {
              _context.next = 6;
              break;
            }

            callback(null, content);
            return _context.abrupt("return");

          case 6:
            _context.prev = 6;
            _context.next = 9;
            return patchComponent(new _magicString["default"](content), component, loaderOptions, function (path) {
              return new Promise(function (resolve) {
                try {
                  _this.resolve(_this.rootContext || _this.options.context, path, function (err) {
                    if (err) {
                      resolve(false);
                      return;
                    }

                    resolve(true);
                  });
                } catch (e) {
                  resolve(false);
                }
              });
            });

          case 9:
            result = _context.sent;
            callback(null, result.toString(), this.sourceMap ? result.generateMap({
              file: _path["default"].basename(this.resourcePath)
            }) : null);
            _context.next = 17;
            break;

          case 13:
            _context.prev = 13;
            _context.t0 = _context["catch"](6);
            console.error(_context.t0);
            callback(_context.t0);

          case 17:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, this, [[6, 13]]);
  }));
  return _ref.apply(this, arguments);
}

function processSync(content, file, options, resolveSync) {
  var component = resolveComponent(file, options);

  if (!component) {
    return content;
  }

  return patchComponentSync(content, component, options, resolveSync);
}
/**
 * Patch the original .vue file with additional peer modules.
 * @param {MagicString} content .vue file content
 * @param {string} component Component name
 * @param {Object} options veui-loader options
 * @param {function} resolve webpack resolve function to see if target peer exists
 * @returns {Promise<MagicString>} A promise that resolved with the patched content
 */


function patchComponent(_x2, _x3, _x4, _x5) {
  return _patchComponent.apply(this, arguments);
}
/**
 * Patch the original .vue file with additional peer modules.
 * @param {MagicString} content .vue file content
 * @param {string} component Component name
 * @param {Object} options veui-loader options
 * @param {function} resolveSync custom synchronous resolve function to see if target peer exists
 * @returns {MagicString} The patched content
 */


function _patchComponent() {
  _patchComponent = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3(content, component, options, resolve) {
    var parts;
    return _regenerator["default"].wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            parts = getParts(component, options);
            _context3.next = 3;
            return Promise.all([].concat((0, _toConsumableArray2["default"])(parts.script), (0, _toConsumableArray2["default"])(parts.style)).map( /*#__PURE__*/function () {
              var _ref7 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2(module) {
                return _regenerator["default"].wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        _context2.next = 2;
                        return assurePath(module.path, resolve);

                      case 2:
                        module.valid = _context2.sent;

                      case 3:
                      case "end":
                        return _context2.stop();
                    }
                  }
                }, _callee2);
              }));

              return function (_x8) {
                return _ref7.apply(this, arguments);
              };
            }()));

          case 3:
            return _context3.abrupt("return", patchContent(content, parts));

          case 4:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _patchComponent.apply(this, arguments);
}

function patchComponentSync(content, component, options, resolveSync) {
  var parts = getParts(component, options);
  var modules = [].concat((0, _toConsumableArray2["default"])(parts.script), (0, _toConsumableArray2["default"])(parts.style));
  modules.forEach(function (module) {
    module.valid = assurePathSync(module.path, resolveSync);
  });
  return patchContent(content, parts);
}
/**
 * Extract potentially dependent parts for a component
 *
 * @param {string} component Component name
 * @param {Object} options veui-loader options
 * @returns {Object} Extracted parts metadata
 */


function getParts(component, options) {
  var _options$alias = options.alias,
      alias = _options$alias === void 0 ? VEUI_PACKAGE_NAME : _options$alias,
      _options$modules = options.modules,
      modules = _options$modules === void 0 ? [] : _options$modules,
      pack = options["package"],
      _options$path = options.path,
      packPath = _options$path === void 0 ? COMPONENTS_DIRNAME : _options$path,
      transform = options.transform,
      fileName = options.fileName,
      locale = options.locale,
      _options$global = options.global,
      global = _options$global === void 0 ? [] : _options$global;

  if (pack && fileName) {
    modules.push({
      "package": pack,
      path: packPath,
      transform: transform,
      fileName: fileName
    });
  }

  if (locale !== false) {
    if (!locale) {
      locale = 'zh-Hans';
    }

    if (!Array.isArray(locale)) {
      locale = [locale];
    }

    locale = locale.filter(function (l) {
      return typeof l === 'string';
    });
    modules = locale.map(function (l) {
      return {
        "package": alias,
        path: "locale/".concat(l),
        transform: false,
        fileName: '{module}.js'
      };
    }).concat(modules);
    global = locale.map(function (l) {
      return {
        path: "".concat(alias, "/locale/").concat(l, "/common.js")
      };
    }).concat(global.map(function (path) {
      return {
        path: path
      };
    }));
  }

  return modules.reduce(function (acc, _ref2) {
    var pack = _ref2["package"],
        _ref2$path = _ref2.path,
        packPath = _ref2$path === void 0 ? COMPONENTS_DIRNAME : _ref2$path,
        transform = _ref2.transform,
        fileName = _ref2.fileName;
    var peerComponent = getPeerFilename(component, {
      transform: transform,
      template: fileName
    });
    var peerPath = (0, _slash["default"])(_path["default"].join(pack, packPath, peerComponent));
    pushPart(acc, {
      path: peerPath
    });
    return acc;
  }, {
    script: (0, _toConsumableArray2["default"])(global),
    style: []
  });
}
/**
 * Patch content with extracted parts metadata
 *
 * @param {MagicString} content Module content
 * @param {Object} parts Extracted parts metadata
 * @returns {MagicString} Patched content
 */


function patchContent(content, parts) {
  Object.keys(parts).forEach(function (type) {
    var paths = parts[type].filter(function (_ref3) {
      var valid = _ref3.valid;
      return valid;
    }).map(function (_ref4) {
      var path = _ref4.path;
      return path;
    });
    return patchType(content, type, paths);
  });
  return content;
}
/**
 * Push peer file dependency into collected parts.
 * @param {Object} parts Collected parts containing scripts and styles
 * @param {Object} file The file to be appended
 */


function pushPart(parts, file) {
  var ext = getExtname(file.path);
  var type = Object.keys(EXT_TYPES).find(function (key) {
    return EXT_TYPES[key].includes(ext);
  });
  parts[type.toLowerCase()].push(file);
}
/**
 * Get extension name of a file
 * @param {string} file File path
 * @returns {string} File extension
 */


function getExtname(file) {
  return _path["default"].extname(file).replace(/\./g, '').toLowerCase();
}

var RE_SCRIPT = /<script(?:\s[^<>]*)?>/i;
/**
 * Patch file content according to a given type.
 * @param {MagicString} content Original content
 * @param {string} type Peer type, can be `script` or `style`
 * @param {Array<string>} peerPaths Peer module paths
 * @returns {MagicString} The patched content
 */

function patchType(content, type, peerPaths) {
  var code = content.toString();
  var normalizedPaths = peerPaths.map(function (path) {
    return (0, _slash["default"])((0, _utils.normalize)(path));
  });

  switch (type) {
    case 'script':
      var scriptImports = normalizedPaths.map(function (path) {
        return "import '".concat(path, "'\n");
      });
      code.replace(RE_SCRIPT, function (match, offset) {
        var replacement = "".concat(match, "\n").concat(scriptImports.join(''));
        content.overwrite(offset, offset + match.length, replacement);
        return replacement;
      });
      break;

    case 'style':
      var styleImports = normalizedPaths.map(function (path) {
        var langStr = '';
        var ext = getExtname(path);

        if (ext !== 'css') {
          langStr = "lang=\"".concat(ext, "\" ");
        }

        return "<style ".concat(langStr, "src=\"").concat(path, "\"></style>\n");
      });
      content.append(styleImports.join(''));
      break;

    default:
      break;
  }

  return content;
}
/**
 * To test the target peer path exists or not.
 * @param {string} modulePath Peer module path
 * @param {function} resolve webpack module resolver
 * @returns {Promise<boolean>} A promise resolved with true if the target peer path exists
 */


function assurePath(_x6, _x7) {
  return _assurePath.apply(this, arguments);
}
/**
 * To test the target peer path exists or not synchronously.
 * @param {string} modulePath Peer module path
 * @param {function} resolveSync webpack module resolver
 * @returns {boolean} True if the target peer path exists
 */


function _assurePath() {
  _assurePath = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee4(modulePath, resolve) {
    return _regenerator["default"].wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            if (!(resolveCache[modulePath] === false)) {
              _context4.next = 4;
              break;
            }

            return _context4.abrupt("return", false);

          case 4:
            if (modulePath in resolveCache) {
              _context4.next = 15;
              break;
            }

            if (!(typeof resolve === 'function')) {
              _context4.next = 15;
              break;
            }

            _context4.prev = 6;
            _context4.next = 9;
            return resolve(modulePath);

          case 9:
            resolveCache[modulePath] = !!_context4.sent;
            _context4.next = 15;
            break;

          case 12:
            _context4.prev = 12;
            _context4.t0 = _context4["catch"](6);
            resolveCache[modulePath] = false;

          case 15:
            return _context4.abrupt("return", resolveCache[modulePath]);

          case 16:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4, null, [[6, 12]]);
  }));
  return _assurePath.apply(this, arguments);
}

function assurePathSync(modulePath, resolveSync) {
  if (resolveCache[modulePath] === false) {
    return false;
  } else if (!(modulePath in resolveCache)) {
    if (typeof resolveSync === 'function') {
      try {
        resolveCache[modulePath] = !!resolveSync(modulePath);
      } catch (e) {
        resolveCache[modulePath] = false;
      }
    }
  }

  return resolveCache[modulePath];
}
/**
 * Convert a component name according to file name template.
 * @param {string} name Peer module file
 * @param {Object} options Transform options
 * @param {string} options.transform Transform type for base name
 * @param {string} options.template File name template
 * @returns {string} Peer module file name
 */


function getPeerFilename(name, _ref5) {
  var _ref5$transform = _ref5.transform,
      transform = _ref5$transform === void 0 ? 'kebab-case' : _ref5$transform,
      _ref5$template = _ref5.template,
      template = _ref5$template === void 0 ? '{module}.css' : _ref5$template;

  if (!name) {
    return null;
  }

  switch (transform) {
    case 'kebab-case':
      name = (0, _utils.kebabCase)(name);
      break;

    case 'camelCase':
      name = (0, _utils.camelCase)(name);
      break;

    case 'PascalCase':
      name = (0, _utils.pascalCase)(name);
      break;

    case false:
    default:
      break;
  }

  return template.replace(/\$?\{module\}/g, name);
}
/**
 * Resolve the underlying component for a given file path.
 * '/dev/veui/src/components/Button.vue' → 'Button'
 * @param {string} file Absolute file path
 * @param {Object} options veui-loader options
 * @returns {?string} The resolved component name (`null` if not a VEUI component)
 */


function resolveComponent(file, options) {
  var _options$alias2 = options.alias,
      alias = _options$alias2 === void 0 ? VEUI_PACKAGE_NAME : _options$alias2; // make sure relative paths resolved to somewhere inside a correct VEUI

  var pkg = _pkgDir["default"].sync(file);

  var isVEUI = (0, _utils.getJSON)(_path["default"].join(pkg, 'package.json')).name === VEUI_PACKAGE_NAME;

  if (!pkg || // not VEUI package
  !isVEUI || // is dep but dep name isn't correct
  _path["default"].basename(_path["default"].dirname(pkg)) === 'node_modules' && _path["default"].basename(pkg) !== alias) {
    return null;
  } // veui/${dir} or veui/src/${dir}


  var dirPath = _path["default"].join(pkg, COMPONENTS_DIRNAME); // runtime


  if (!_fs["default"].existsSync(dirPath)) {
    dirPath = _path["default"].join(pkg, 'src', COMPONENTS_DIRNAME); // dev

    if (!_fs["default"].existsSync(dirPath)) {
      return null;
    }
  } // is VEUI component


  return getComponentName(_path["default"].relative(dirPath, file), options);
}
/**
 * Convert a component relative path to a component name according to
 * VEUI's component list.
 * 'Icon.vue' → 'Icon'
 * @param {string} componentPath Component path
 * @param {Object} options veui-loader options
 * @returns {?string} Component name (`null` if not a VEUI component)
 */


function getComponentName(componentPath, options) {
  var _options$alias3 = options.alias,
      alias = _options$alias3 === void 0 ? VEUI_PACKAGE_NAME : _options$alias3;

  var components = require("".concat(alias, "/components.json"));

  if (!componentPath) {
    return null;
  }

  var component = components.find(function (_ref6) {
    var path = _ref6.path;
    path = (0, _utils.normalize)(path);
    return path === componentPath || path.split('.')[0] === componentPath;
  });
  return component ? component.name : null;
}